import java.util.*;

class moveList {
	int halfmoves;
	ArrayList board;
	ArrayList move;
	ArrayList color;

	moveList(){
	    board=new ArrayList();
	    move=new ArrayList();
	    color=new ArrayList();
	    halfmoves=0;
	}

	public String getMove(int n){
	    return (String)move.get(n);
	}

	public char[] getBoard(int n){
	    char[] r = new char[64];
	    char[] g = (char[])board.get(n);
	    for(int i=0;i<64;i++)
		r[i]=g[i];
	    return r;
	}

	public int getHalfmoves() { return halfmoves; }

	int coords2square(String c){
	    if(c.length()!=2) return -1;
	    int x=c.charAt(0)-'a';
	    int y=c.charAt(1)-'1';
	    if(y>7 || y<0 || x>7 || x<0) return -1;
	    return (7-y)*8+x;
	}

    public void clear(){
	board.clear();
	move.clear();
	color.clear();
	halfmoves=0;
    }

	public void add(String m,char[] b){
	    board.add(b);
	    move.add(m);
	    if(((String)color.get(color.size()-1)).equals("White"))
		color.add("Black");
	    else
		color.add("White");
	    halfmoves++;
	}

	public void set(String s){
	    board.clear();
	    move.clear();
	    color.clear();
	    halfmoves=0;

	    move.add("");
	    board.add(fen2board("rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR"));
	    color.add("Black");

	    int c=0;
	    
	    while(c<s.length()){
		
		if(s.charAt(c)=='.' && c+1<s.length()){
		    c++;
		    
		    while(s.charAt(c)==' ' && c+1<s.length())
			c++;
		    
		    int ws=c;
		    while(s.charAt(c)!=' ' && c+1<s.length())
			c++;

		    String w=s.substring(ws,c);
		    
		    while(s.charAt(c)==' ' && c+1<s.length())
			c++;

		    int bs=c;
		    while(s.charAt(c)!=' ' && c+1<s.length())
			c++;
		    String b=s.substring(bs,c);

		    if(w.length()>1) {
			move.add(w);
			halfmoves++;
			color.add("White");
		    }
		    if(b.length()>1){ 
			move.add(b);
			halfmoves++;
			color.add("Black");
		    }
		    
		} 
		  
		c++;
	    }
	    /** Set boards **/
	    for(int n=1;n<=halfmoves;n++){
		char[] newboard = getBoard(n-1);
		String m=getMove(n);
		String mc=(String)color.get(n);

		tofrom t=new tofrom(newboard, m,mc);

		board.add(t.getBoard());
	    }
	
	    
	}


	char getPieceAt(char[] b, int x, int y){
	    if(x<0 || x>7 || y<0 || y>7)
		return 0;
	    return b[y*8+x];
	}


	/** Is there a night on a night-distance from square? **/
	int findNightLength(char[] b, int x, int y,char n){
	    if(getPieceAt(b,x-1,y-2) == n) return x-1+(y-2)*8;
	    if(getPieceAt(b,x+1,y-2) == n) return x+1+(y-2)*8;
	    if(getPieceAt(b,x-2,y-1) == n) return x-2+(y-1)*8;
	    if(getPieceAt(b,x+2,y-1) == n) return x+2+(y-1)*8;
	    if(getPieceAt(b,x-2,y+1) == n) return x-2+(y+1)*8;
	    if(getPieceAt(b,x+2,y+1) == n) return x+2+(y+1)*8;
	    if(getPieceAt(b,x-1,y+2) == n) return x-1+(y+2)*8;
	    if(getPieceAt(b,x+1,y+2) == n) return x+1+(y+2)*8;

	    return -1;
	}

	int findKingLength(char[] b, int x, int y,char n){
	    if(getPieceAt(b,x-1,y-1) == n) return x-1+(y-1)*8;
	    if(getPieceAt(b,x,  y-1) == n) return x+(y-1)*8;
	    if(getPieceAt(b,x+1,y-1) == n) return x+1+(y-1)*8;

	    if(getPieceAt(b,x-1,y) == n) return x-1+(y)*8;
	    if(getPieceAt(b,x+1,y) == n) return x+1+(y)*8;

	    if(getPieceAt(b,x-1,y+1) == n) return x-1+(y+1)*8;
	    if(getPieceAt(b,x,  y+1) == n) return x+(y+1)*8;
	    if(getPieceAt(b,x+1,y+1) == n) return x+1+(y+1)*8;

	    return -1;
	}

    int findRookLength(char[] b, int x, int y, char r){
	int xx=0; 
	int yy=0;

	do{
	    xx++;
	    if(getPieceAt(b,x+xx,y+yy) == r) return x+xx+(y+yy)*8;
	}while(getPieceAt(b,x+xx,y+yy) == '.');

	xx=0;
	do{
	    xx--;
	    if(getPieceAt(b,x+xx,y+yy) == r) return x+xx+(y+yy)*8;
	}while(getPieceAt(b,x+xx,y+yy) == '.');

	xx=0;
	do{
	    yy--;
	    if(getPieceAt(b,x+xx,y+yy) == r) return x+xx+(y+yy)*8;
	}while(getPieceAt(b,x+xx,y+yy) == '.');

	yy=0;
	do{
	    yy++;
	    if(getPieceAt(b,x+xx,y+yy) == r) return x+xx+(y+yy)*8;
	}while(getPieceAt(b,x+xx,y+yy) == '.');

	return -1;
    }

    int findBishopLength(char[] b,int x, int y, char r){
	int xx=0; 
	int yy=0;

	do{
	    xx++;
	    yy++;
	    if(getPieceAt(b,x+xx,y+yy) == r) return x+xx+(y+yy)*8;
	}while(getPieceAt(b,x+xx,y+yy) == '.');

	xx=0;
	yy=0;
	do{
	    xx--;
	    yy--;
	    if(getPieceAt(b,x+xx,y+yy) == r) return x+xx+(y+yy)*8;
	}while(getPieceAt(b,x+xx,y+yy) == '.');

	xx=0;
	yy=0;
	do{
	    yy--;
	    xx++;
	    if(getPieceAt(b,x+xx,y+yy) == r) return x+xx+(y+yy)*8;
	}while(getPieceAt(b,x+xx,y+yy) == '.');

	yy=0;
	xx=0;
	do{
	    yy++;
	    xx--;
	    if(getPieceAt(b,x+xx,y+yy) == r) return x+xx+(y+yy)*8;
	}while(getPieceAt(b,x+xx,y+yy) == '.');

	return -1;
    }

    int findQueenLength(char[] b, int x, int y, char q){
	if(findRookLength(b,x,y,q)>=0) return findRookLength(b,x,y,q);
	if(findBishopLength(b,x,y,q)>=0) return findBishopLength(b,x,y,q);

	return -1;
    }

    int findPawnLength(char[] b, int x, int y, char p){
	if(p=='P'){
	    if(getPieceAt(b,x,y+1)==p) return x+(y+1)*8;
	    if(getPieceAt(b,x,y+2)==p && y==4) return x+(y+2)*8;
	}
	else {
	    if(getPieceAt(b,x,y-1)==p) return x+(y-1)*8;
	    if(getPieceAt(b,x,y-2)==p && y==3) return x+(y-2)*8;
	}

	return -1;
    }


    char[] fen2board(String aFen){
	char[] board = new char[64];
	int square=0;
	int chr=0;

	while(square<64){
	    char c = aFen.charAt(chr);
	    if(c=='r'||c=='R'||c=='n'||c=='N'||c=='b'||c=='B'||c=='q'||c=='Q'||c=='k'||c=='K'||c=='p'||c=='P'){
		board[square]=c;
		square++;
	    }
	    if(c>='1' && c<='8'){
		for(int i=0;i<c-'1'+1;i++){
		    board[square]='.';
		    square++;
		}
	    }
	    chr++;
	}
	return board;
    }



class tofrom {
    char piece;
    char[] board;

    public tofrom() { }
    public tofrom(char[] newboard, String m, String mc) { calc(newboard, m,mc); }

    public char[] getBoard() { return board; }
    
 
    public void calc(char[] newboard, String m, String mc){
	board=newboard;

	if(m.length()<2) return;

	/** Castling **/
	if(m.equals("O-O") && mc.equals("White")){
	    board[60]='.'; board[62]='K'; board[63]='.'; board[61]='R';
	    return;
	}
	if(m.equals("O-O-O") && mc.equals("White")){
	    board[60]='.'; board[58]='K'; board[56]='.'; board[59]='R';
	    return;
	}
	if(m.equals("O-O") && mc.equals("Black")){
	    board[4]='.'; board[6]='K'; board[7]='.'; board[5]='R';
	    return;
	}
	if(m.equals("O-O-O") && mc.equals("Black")){
	    board[4]='.'; board[2]='K'; board[0]='.'; board[3]='R';
	    return;
	}

        int toSquare=-1;
        int fromSquare=-1;

	/** Piece **/
	piece=m.charAt(0);
	if(piece<'A' || piece>'Z')
	    piece='P';
	if( mc.equals("Black"))
	    piece = (char)(piece-'A'+'a');

	/** Remove check (+) and everything else after **/
	int last=m.length()-1;
	while(m.length()>1 && (m.charAt(last) < '1' || m.charAt(last) > '8')){
	    m=m.substring(0,m.length()-1);
	    last=m.length()-1;
	}

	if(m.length()<2) return;

	/** More pieces to square **/
	if(m.length()>4){
	    if( m.charAt(2)>='1' && m.charAt(2)<='8'){
		fromSquare=coords2square(m.substring(1,3));
	    }
	}

	/** To square **/
	toSquare=coords2square(m.substring(m.length()-2));

	/** Maybe we already have the move now **/
	if(toSquare>=0 && fromSquare>=0){
	    board[toSquare]=piece;
	    board[fromSquare]='.';
	    return;
	}

	fromSquare=0;

	/** The normal fromSquare rutine **/
	if(piece=='P' || piece=='p'){
	    if(m.charAt(1)=='x' && m.length()==4){
		if(piece=='P') fromSquare = (8-(int)(m.charAt(3)-'1'))*8+(int)(m.charAt(0)-'a');
		if(piece=='p') fromSquare = (6-(int)(m.charAt(3)-'1'))*8+(int)(m.charAt(0)-'a');
	    }
	    else
		fromSquare=findPawnLength(newboard,toSquare%8,toSquare/8,piece);
	}

	if(piece=='B' || piece=='b')
	    fromSquare=findBishopLength(newboard,toSquare%8,toSquare/8,piece);
	if(piece=='N' || piece=='n')
	    fromSquare=findNightLength(newboard,toSquare%8,toSquare/8,piece);
	if(piece=='R' || piece=='r')
	    fromSquare=findRookLength(newboard,toSquare%8,toSquare/8,piece);
	if(piece=='Q' || piece=='q')
	    fromSquare=findQueenLength(newboard,toSquare%8,toSquare/8,piece);
	if(piece=='K' || piece=='k')
	    fromSquare=findKingLength(newboard,toSquare%8,toSquare/8,piece);

	/** Move! **/
	if(toSquare>=0 && toSquare<64 && fromSquare>=0 && fromSquare<64){
	    board[toSquare]=piece;
	    board[fromSquare]='.';
	}


    }

}

}













